@@ -429,7 +429,8 @@ namespace BOOM {
     const Vector &v(marg.prediction_error());
 
     Ptr<SparseKalmanMatrix> Finv = marg.sparse_forecast_precision();
-    Ptr<SparseMatrixProduct> K(marg.sparse_kalman_gain(observed_status(t)));
+    Ptr<SparseMatrixProduct> K(marg.sparse_kalman_gain(
+        observed_status(t), Finv));
 
     Vector observation_error_mean = H * (*Finv * v - *K * r);
     Vector observation_error_variance =
@@ -458,4 +459,13 @@ namespace BOOM {
     report_error("CindBase::update_observation_model isn't done.");
   }
 
+  Matrix MvBase::state_mean() const {
+    const auto &kalman_filter(get_filter());
+    Matrix ans(state_dimension(), time_dimension());
+    for (size_t i = 0; i < time_dimension(); ++i) {
+      ans.col(i) = kalman_filter[i].state_mean();
+    }
+    return ans;
+  }
+
 }  // namespace BOOM
